/*
 * Copyright 2014-2022 Univention GmbH
 *
 * https://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <https://www.gnu.org/licenses/>.
 */
/*global define*/

define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "login",
    "umc/menu",
    "umc/tools",
    "umc/dialog",
    "umc/widgets/Text",
    "umc/widgets/PasswordBox",
    "umc/widgets/PasswordInputBox",
    "umc/widgets/Page",
    "dijit/Dialog",
    "umc/widgets/Form",
    "dijit/form/Button",
    "umc/widgets/Image",
    "umc/widgets/Module",
    "umc/i18n!umc/modules/twofactor-user",
], function (declare, lang, topic, login, menu, tools, dialog, Text, PasswordBox, PasswordInputBox, Page, Dialog, Form, Button, Image, Module, _) {
    return declare("umc.modules.twofactor-user", [Module], {
        newTokenDialog: null,

        generateToken() {
            tools.umcpCommand('twofactor-user/generate_inital_qr_token', {}, false).then(lang.hitch(this, function (ret) {
                this.newTokenDialog.hide()
                this.newTokenDialog.destroy()

                if (!ret['result'].hasOwnProperty("image")) {
                    umcdialog.confirm(_("Error while fetching the QR code image"),
                        [{
                            'default': true,
                            label: _('OK')
                        }]
                    )
                } else {
                    var form = new Form();
                    var qrcode = new Image({
                        id: "qrcode",
                        value: ret['result']['image'].replace("data:image/png;base64,", ""),
                        style: "background: none; max-width:100%; max-height:100%;",
                        'data-qa': 'qrcode-image'
                    })

                    qrcode.placeAt(form.containerNode)

                    new Button({
                        label: _("Close"),
                        'data-qa': 'closebutton',
                        onClick: lang.hitch(this, function (ret) {
                            qrcode.destroy()
                            myDialog.hide()
                            window.location.replace("/univention/portal");
                            1
                        })
                    }, "progbutton").placeAt(form.containerNode);

                    var myDialog = new Dialog({
                        title: _("Please scan the QR code with the two-factor application on your phone."),
                        style: "width: 100%; background: none",
                        content: form
                    });
                    myDialog.show()
                }
            }))
        },

        generateNewTokenDialog() {

            var warning = _("<p style=\"color:#ff0000;\">You can only generate a software token for two-factor authentication yourself <b>once</b>. If you do not want to use an app "
							  + "for two-factor authentication on your tablet/smartphone, please click <b>&quot;Cancel&quot;</b> and contact your school administrator.</p>")

            var introText = _("If you choose to use the app, please install one of the following apps on your tablet or smartphone:<br />"
							  + "<ul>"
							  + "<li>andOTP</li>"
							  + "<li>Aegis Authenticator</li>"
							  + "<li>MS Authenticator</li>"
							  + "</ul>"
							  + "If you want to set up the software token later, please click <b>&quot;Cancel&quot;</b>. If you have installed an app and it is ready to scan a token, "
							  + "please click <b>&quot;Continue&quot;</b>."
							 )

            var text = "<table>\n" +
                "    <tr>\n" +
                "        <td colspan=\"2\">\n" +
                "            <p>" + warning + "</p>\n" +
                "            <p>" + introText + "</p>\n" +
                "        </td>\n" +
                "    </tr>\n" +
                "</table>"


            var buttons = [{
                type: Button,
                label: _("Cancel"),
                name: 'newtoken_cancel',
                onClick: lang.hitch(this, function () {
                    this.newTokenDialog.hide()
                    this.newTokenDialog.destroy()
                    window.location.replace("/univention/portal");
                })
            },
                {
                    type: Button,
                    label: _("Continue"),
                    onClick: lang.hitch(this, this.generateToken),
                    name: 'newtoken_submit'
                }
            ]


            var form = new Form({
                buttons: buttons,
                fullWidth: true,
                widgets: [{
                    type: Text,
                    content: text,
                    name: "test"
                }]
            });

            this.newTokenDialog = new Dialog({
                title: _("Please install a two-factor application on your smartphone"),
                style: "width: 100%; dgrid-content",
                content: form
            });
        },

        initalTokenDialog() {
            menu.close();

            tools.umcpCommand('twofactor-user/check_token_status', {}, false).then(lang.hitch(this, function (ret) {
                // either show token info or manual for token generation and matching button
                if (ret['result']['state'] == "user_has_token(s)") {
                    dialog.confirm(_('A hardware or software token has already been created for your user account. For security reasons, '
									 + 'only one token can be issued per user account. If you have any problems with your hardware or software token, '
									 + 'please contact your school administrator.'), [{
                            label: _('OK'),
                            callback: function () {
                                window.location.replace("/univention/portal")
                            }
                        }]
                    );
                } else if (ret['result']['state'] == "user_has_no_token(s)") {
                    this.generateNewTokenDialog()
                    this.newTokenDialog.show()
                } else {
                    dialog.alert("ERROR")
                }

            }), lang.hitch(this, function (err) {
                err = tools.parseError(err);
                dialog.confirm(err.message, [{
                    label: _('OK'),
                    'default': true
                }], _('Error'));
            }));
        },

        buildRendering: function () {
            this.inherited(arguments);
            this.initalTokenDialog()
        }
    });
});
